@@ -654,6 +654,7 @@ def validate_prop_info_list(prop_info_list, pos_list, feature):
     """
     Perform various checks on a list of properties in a property-block
         - make sure that properties that should appear first (substitute type) appear first
+        - make sure that required properties are set
 
     @param prop_info_list: List of dictionaries with property information
     @type prop_info_list: C{list} of C{dict}
@@ -665,15 +666,23 @@ def validate_prop_info_list(prop_info_list, pos_list, feature):
     @type feature: C{int}
     """
     first_warnings = [(info, pos_list[i]) for i, info in enumerate(prop_info_list) if "first" in info and i != 0]
-    for info, pos in first_warnings:
-        for prop_name, prop_info in properties[feature].items():
-            if info == prop_info or (isinstance(prop_info, list) and info in prop_info):
+    for prop_name, prop_info in properties[feature].items():
+        if not isinstance(prop_info, list):
+            prop_info = [prop_info]
+        for info, pos in first_warnings:
+            if info in prop_info:
                 generic.print_warning(
                     generic.Warning.GENERIC,
                     "Property '{}' should be set before all other properties and graphics.".format(prop_name),
                     pos,
                 )
                 break
+        for info in prop_info:
+            if "required" in info and info not in prop_info_list:
+                generic.print_error(
+                    "Property '{}' is not set. Item will be invalid.".format(prop_name),
+                    pos_list[-1],
+                )
 
 
 def parse_property_block(prop_list, feature, id, size):
@@ -717,7 +726,7 @@ def parse_property_block(prop_list, feature, id, size):
         value_list_list.extend(
             parse_property_value(prop_info, prop.value, prop.unit, size_bit) for prop_info in new_prop_info_list
         )
-        pos_list.extend(prop.name.pos for i in prop_info_list)
+        pos_list.extend(prop.name.pos for i in new_prop_info_list)
 
     validate_prop_info_list(prop_info_list, pos_list, feature)
 
@@ -1074,6 +1083,7 @@ callback_flag_properties = {
     0x01: {"size": 1, "num": 0x17},
     0x02: {"size": 1, "num": 0x12},
     0x03: {"size": 1, "num": 0x14},
+    0x04: {"size": 1, "num": 0x0B},
     0x05: {"size": 1, "num": 0x08},
     0x07: two_byte_property(0x14, 0x1D),
     0x09: {"size": 1, "num": 0x0E},
@@ -1152,3 +1162,48 @@ def get_volume_actions(volume_list):
                 value_series = [volume]
 
     return action_list
+
+
+class StationLayoutProp(BaseAction0Property):
+    def __init__(self, layouts):
+        self.layouts = layouts
+
+    def write(self, file):
+        file.print_bytex(0x1A)
+        file.print_byte(len(self.layouts))
+        file.newline()
+        for layout in self.layouts:
+            layout.write(file)
+
+    def get_size(self):
+        size = 2
+        for layout in self.layouts:
+            size += layout.get_size()
+        return size
+
+
+def get_layout_action0(feature, id, layouts):
+    action_list = []
+    act0, offset = create_action0(feature, id, None, None)
+    action6.free_parameters.save()
+    act6 = action6.Action6()
+
+    for layout in layouts:
+        layout.write_action_value(action_list, act6, offset)
+
+    if len(act6.modifications) > 0:
+        action_list.append(act6)
+
+    act0.num_ids = 1
+    act0.prop_list.append(StationLayoutProp(layouts))
+    action_list.append(act0)
+
+    action6.free_parameters.restore()
+    return action_list
+
+
+def get_copy_layout_action0(feature, id, source_id):
+    act0, offset = create_action0(feature, id, None, None)
+    act0.num_ids = 1
+    act0.prop_list.append(Action0Property(0x0A, source_id, 1))
+    return [act0]
