@@ -217,10 +217,10 @@ def free_references(source_action):
             free_action2_ids.append(act2.id)
 
 
-# Features using sprite groups directly: vehicles, canals, cargos, railtypes, airports, roadtypes, tramtypes
-features_sprite_group = [0x00, 0x01, 0x02, 0x03, 0x05, 0x0B, 0x0D, 0x10, 0x12, 0x13]
-# Features using sprite layouts: stations, houses, industry tiles, objects and airport tiles
-features_sprite_layout = [0x04, 0x07, 0x09, 0x0F, 0x11]
+# Features using sprite groups directly: vehicles, stations, canals, cargos, railtypes, airports, roadtypes, tramtypes
+features_sprite_group = [0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x0B, 0x0D, 0x10, 0x12, 0x13]
+# Features using sprite layouts: houses, industry tiles, objects and airport tiles
+features_sprite_layout = [0x07, 0x09, 0x0F, 0x11]
 # All features that need sprite sets
 features_sprite_set = features_sprite_group + features_sprite_layout
 
