@@ -128,6 +128,10 @@ class Switch(switch_base_class):
         return all_refs
 
     def is_read_only(self):
+        for result in [r.result for r in self.body.ranges] + [self.body.default]:
+            if result is not None and result.value is not None:
+                if not result.value.is_read_only():
+                    return False
         return self.expr.is_read_only()
 
     def debug_print(self, indentation):
@@ -307,7 +311,9 @@ class RandomSwitch(switch_base_class):
             if len(self.type.params) == 0:
                 self.type_count = None
             elif len(self.type.params) == 1:
-                self.type_count = action2var.reduce_varaction2_expr(self.type.params[0], feature)
+                # var_scope is really weird for type=BACKWARD/FORWARD.
+                var_scope = action2var.get_scope(feature, 0x89)
+                self.type_count = action2var.reduce_varaction2_expr(self.type.params[0], var_scope)
             else:
                 raise generic.ScriptError(
                     "Value for random_switch parameter 2 'type' can have only one parameter.", self.type.pos
